/* xdwapi sample 14

Function :
  Set every updatable annotation in a DocuWorks file with a unique ID, and then generates an XML file from the DocuWorks file.
  Use the XML file generated from "updateGet" to update the DocuWorks file.

How to use :
  xdwsmpl14 updateGet|updateSet docuWorksDocument xmlDocument

  updateGetFSet every updatable annotation in a DocuWorks file with a unique ID, and then generates an XML file from the DocuWorks file.
  updateSetFUse the XML file generated from "updateGet" to update the DocuWorks file.
  docuWorksDocument : DocuWorks document
  xmlDocument : XML document
*/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>

#include <xdw_api.h>
#include <xdw_xml.h>

// User-defined attribute name to identify the annotations to be updated
#define XDWSMPL_USER_DEFINE_ATTRIBUTE_NAME "UpdateAnnotationMark"

// Display what cause an error.
void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_FILE_EXISTS:
		fprintf(stderr, "File exists.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_ACCESS:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

// Dispaly a help message.
void print_help()
{
	fprintf(stderr, "How to use : xdwsmpl14 updateGet|updateSet docuWorksDocument xmlDocument");
}

// Set a unique user-defined attribute to a specified annotation.
int set_annotation_user_attribute(XDW_DOCUMENT_HANDLE handle_xdw_document, XDW_ANNOTATION_HANDLE handle_annotation, int annotation_type, const char* attribute_name)
{
	int api_result = XDW_E_INVALIDARG;
	if (handle_xdw_document != NULL && handle_annotation != NULL && attribute_name != NULL && *attribute_name != NULL) {
		bool api_supported_type = FALSE;
		api_result = 0;
		// only those annotation can be changed by XDWAPI are subject to be setted
		switch (annotation_type) {
			case XDW_AID_FUSEN:
			case XDW_AID_TEXT:
			case XDW_AID_STAMP:
			case XDW_AID_RECTANGLE:
			case XDW_AID_ARC:
			case XDW_AID_LINK:
				api_supported_type = TRUE;
				break;
		}
		if (api_supported_type) {
			// use a GUID as a unique ID 
			GUID guid = {0};
			HRESULT hResult = CoCreateGuid(&guid);
			api_result = (int)hResult;
			if (SUCCEEDED(hResult)) {
			api_result = XDW_SetAnnotationUserAttribute(handle_xdw_document, handle_annotation, attribute_name, (char*)&guid, sizeof(GUID), NULL);
			}
		}
	}
	return api_result;
}

// Set unique user-defined attribute to every updatable annotation in a DocuWorks document.
int set_annotation_user_attribute(XDW_DOCUMENT_HANDLE handle, const char* attribute_name)
{
	int api_result = XDW_E_INVALIDARG;
	if (handle != NULL && attribute_name != NULL) {
		XDW_DOCUMENT_INFO document_infomation = { sizeof(XDW_DOCUMENT_INFO) };
		api_result = XDW_GetDocumentInformation(handle, &document_infomation);
		// for every page in this document.
		for (int page_index = 1; page_index <= document_infomation.nPages && api_result >= 0; page_index++) {
			XDW_PAGE_INFO page_info = { sizeof(XDW_PAGE_INFO) };
			api_result = XDW_GetPageInformation(handle, page_index, &page_info);
			// for every annotation that can be updated in this page.
			for (int annotation_index = 1; annotation_index <= page_info.nAnnotations && api_result >= 0; annotation_index++) {
				XDW_ANNOTATION_INFO annotation_info = { sizeof(XDW_ANNOTATION_INFO) };
				api_result = XDW_GetAnnotationInformation(handle, page_index, NULL, annotation_index, &annotation_info, NULL);
				if (api_result >= 0) {
					api_result = set_annotation_user_attribute(handle, annotation_info.handle, annotation_info.nAnnotationType, attribute_name);
				}
				// for every child annotation.
				for (int child_index = 1; child_index <= annotation_info.nChildAnnotations && api_result >= 0; child_index++) {
					XDW_ANNOTATION_INFO childAnnoInfo = { sizeof(XDW_ANNOTATION_INFO) };
					api_result = XDW_GetAnnotationInformation(handle, page_index, annotation_info.handle, child_index, &childAnnoInfo, NULL);
					if (api_result >= 0) {
						api_result = set_annotation_user_attribute(handle, childAnnoInfo.handle, childAnnoInfo.nAnnotationType, attribute_name);
					}
				}
			}
		}
	}
	return api_result;
}

//  Set every updatable annotation in a DocuWorks file with a unique ID, and then generates an XML file from the DocuWorks file.
void get_xml_contains_user_attribute(const char* dw_full_path, const char* xml_full_path)
{
	int api_result = 0;

	// Open the document handle.
	// In order to set unique ID to annotation, document should be opened by XDW_OPEN_UPDATE.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX open_mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_UPDATE, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(dw_full_path, &h, (XDW_OPEN_MODE*)&open_mode);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Set unique user-defined attribute to every updatable annotation in a DocuWorks document.
	api_result = set_annotation_user_attribute(h, XDWSMPL_USER_DEFINE_ATTRIBUTE_NAME);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Use XDW_GetXMLFile to Generates an XML file contains user-defined attribute from a DocuWorks file.
	XDW_XML_GET_OPTION_DOCUWORKSFILE get_option = {
		sizeof(XDW_XML_GET_OPTION_DOCUWORKSFILE), XDW_XML_ENCODING_UTF8, 1, 0, 
		XDW_XML_CONVERT_KNOWN_AND_SPECIFIED_CODEPAGE, CP_ACP, 1, 
		XDW_XML_DOCUWORKSFILE_SCHEMA_VERSION_710, XDW_XML_DOCUWORKSFILE_XDWAPI_VERSION_710, XDWSMPL_USER_DEFINE_ATTRIBUTE_NAME};
	api_result = XDW_GetXmlFile(h, xml_full_path, XDW_XML_FORMAT_DOCUWORKSFILE, &get_option, NULL);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Save the user-defined attribute to DocuWorks Document.
	api_result = XDW_SaveDocument(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return ;
	}

	// Close the document handle.
	api_result = XDW_CloseDocumentHandle(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return;
	}
}

// Use the XML file generated from "updateGet" to update the DocuWorks file.
void set_xml_specified_user_attribute(const char* dw_full_path, const char* xml_full_path)
{
	int api_result = 0;

	// Open the document handle.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX open_mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_UPDATE, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(dw_full_path, &h, (XDW_OPEN_MODE*)&open_mode);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Use XDW_SetXmlFile to Update the DocuWorks file with  the XML file generated from "updateGet".
	XDW_XML_SET_OPTION_DOCUWORKSFILE set_option = {
		sizeof(XDW_XML_SET_OPTION_DOCUWORKSFILE), XDW_XML_SET_ANNOTATION_UPDATE, XDWSMPL_USER_DEFINE_ATTRIBUTE_NAME};
	XDW_XML_ERROR_INFO error_info = {
		sizeof(XDW_XML_ERROR_INFO), 0, 0};
	api_result = XDW_SetXmlFile(h, xml_full_path, XDW_XML_FORMAT_DOCUWORKSFILE, &set_option, &error_info, NULL);
	if (api_result < 0) {
		print_error(api_result);
		fprintf(stderr, "XML file has an error located at: Line %d, Column %d.", error_info.nLine, error_info.nColumn);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Reflect the changes to the document.
	api_result = XDW_SaveDocument(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return ;
	}

	// Close the document handle.
	api_result = XDW_CloseDocumentHandle(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return;
	}
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc != 4) {
		print_help();
		return 0;
	}
	
	char dw_full_path[_MAX_PATH] = {0};
	char xml_full_path[_MAX_PATH] = {0};
	
	_fullpath(dw_full_path, argv[2], _MAX_PATH);
	_fullpath(xml_full_path, argv[3], _MAX_PATH);

	if (_strcmpi(argv[1], "updateGet") == 0) {
		get_xml_contains_user_attribute(dw_full_path, xml_full_path);
	}
	else if (_strcmpi(argv[1], "updateSet") == 0) {
		set_xml_specified_user_attribute(dw_full_path, xml_full_path);
	}
	else {
		print_help();
	}

	return 0;
}
