/* xdwapi sample 9

Function :
  Sets the document attributes for DocuWorks document.
  Attaches the label annotation at each page.
  Succeeds attributes of the DocuWorks document to another one.
  Gets the document attributes from another one.

How to use :
  xdwsmpl9 document1 document2

  document1 : DocuWorks document (in).
  document2 : DocuWorks document (in).
*/

#include <stdio.h>
#include <windows.h>

#include <xdw_api.h>
#include <xdwapian.h>

void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	case XDW_E_INVALID_ACCESS:
		fprintf(stderr, "Invalid access.");
		break;
	case XDW_E_OUTOFMEMORY:
		fprintf(stderr, "Out of memory.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

int add_annotation(XDW_DOCUMENT_HANDLE h)
{
	// Use XDW_GetDocumentInformation to get the number of total pages.
	XDW_DOCUMENT_INFO info = { sizeof(XDW_DOCUMENT_INFO), 0 };
	int api_result = XDW_GetDocumentInformation(h, &info);
	if (api_result < 0) return api_result;
	int last_page = info.nPages;

	XDW_ANNOTATION_HANDLE annotation;
	int x = 500;
	int y = 500;
	XDW_AA_FUSEN_INITIAL_DATA label;
	label.common.nSize = sizeof(XDW_AA_FUSEN_INITIAL_DATA);
	label.common.nAnnotationType = XDW_AID_FUSEN;
	label.common.nReserved1 = label.common.nReserved2 = 0;
	label.nWidth = 7500;
	label.nHeight = 2500;
	int color = XDW_COLOR_FUSEN_YELLOW;

	// Attach a label annotation at each page
	for (int i = 1; i <= last_page; i++) {
		api_result = XDW_AddAnnotation(
			h, XDW_AID_FUSEN, i, x, y, (XDW_AA_INITIAL_DATA*)&label, &annotation, NULL);
		if (api_result < 0) break;

		// Set the attribute of the label annotation color.
		api_result = XDW_SetAnnotationAttribute(
			h, annotation, XDW_ATN_FillColor, XDW_ATYPE_INT, (char*)&color, 0, NULL);
		if (api_result < 0) break;
	}

	return api_result;
}

int set_document_attribute(XDW_DOCUMENT_HANDLE h)
{
	int api_result = 0;

	char string[100];
	// Set "Title"(Summary) as the document attribute.
	strcpy(string, "xdwapi sample 9");
	api_result = XDW_SetDocumentAttribute(h, XDW_PROP_TITLE, XDW_ATYPE_STRING, string, NULL);
	if (api_result < 0) return api_result;

	// Set "Owner"(Custom) as the document attribute.
	strcpy(string, "ABCxyz");
	api_result = XDW_SetDocumentAttribute(h, "User", XDW_ATYPE_STRING, string, NULL);
	if (api_result < 0) return api_result;

	// "Document number"(Custom) as the document attribute.
	int number = 1234567890;
	api_result = XDW_SetDocumentAttribute(h, "Document number", XDW_ATYPE_INT, (char*)&number, NULL);

	return api_result;
}

int succeed_attribute(XDW_DOCUMENT_HANDLE h, const char* file_path)
{
	// Set the attribute to be succeeded.
	int succession = XDW_SUMMARY_INFO | XDW_USER_DEF | XDW_ANNOTATION;

	// Succeed the attributes.
	int api_result = XDW_SucceedAttribute(h, file_path, 0, succession, NULL);

	return api_result;
}

int get_document_attribute(XDW_DOCUMENT_HANDLE h)
{
	int api_result = 0;
	int size = 0;
	char attribute_name[256];
	int attribute_type;

	// Get the number of document attributes set.
	int attribute_num = XDW_GetDocumentAttributeNumber(h, NULL);
	for (int i = 1; i <= attribute_num; i++) {
		// Get the byte count required for the allocation of the attribute value.
		size = XDW_GetDocumentAttributeByOrder(h, i, attribute_name, &attribute_type, NULL, 0, NULL);
		if (size > 0) {
			char *attribute_value = new char[size];
			if (attribute_value == NULL) return XDW_E_OUTOFMEMORY;
			// Get the document attribute.
			api_result = XDW_GetDocumentAttributeByOrder(h, i, attribute_name, &attribute_type, attribute_value, size, NULL);
			if (api_result < 0) break;

			if (! stricmp(attribute_name, XDW_PROP_TITLE)) {
				printf(" Title is %s.\n", attribute_value);
			} else {
				if (attribute_type == XDW_ATYPE_STRING) {
					printf(" %s is %s.\n", attribute_name, attribute_value);
				} else if (attribute_type == XDW_ATYPE_INT) {
					printf(" %s is %ld.\n", attribute_name, *((int*)attribute_value));
				}
			}
			delete [] attribute_value;
		}
	}

	return api_result;
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc != 3) {
		fprintf(stderr, "How to use : xdwsmpl9 document1 document2");
		return 0;
	}

	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_UPDATE, XDW_AUTH_NODIALOGUE };

	// Open the document handle of document1.
	char in_path[_MAX_PATH];
	_fullpath(in_path, argv[1], _MAX_PATH);
	api_result = XDW_OpenDocumentHandle(in_path, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Set the document attributes in document1.
	api_result = set_document_attribute(h);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Attach the label annotation.
	api_result = add_annotation(h);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Reflect the changes to document1.
	api_result = XDW_SaveDocument(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	// Open the document handle of document2.
	char in_path2[_MAX_PATH];
	_fullpath(in_path2, argv[2], _MAX_PATH);
	api_result = XDW_OpenDocumentHandle(in_path2, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Succeed the attributes from document1 to document2.
	api_result = succeed_attribute(h, in_path);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Get the document attributes in document2.
	api_result = get_document_attribute(h);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Reflect the changes to document2.
	api_result = XDW_SaveDocument(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	if (api_result < 0) return 0;

	return 1;
}