/* xdwapi sample 8

Function :
  Sets the attributes of headers, footers and page-numbers to the DocuWorks document and binder
  and updates them.
How to use :
  xdwsmpl8 document

  document : DocuWorks document (in).

  Creates DocuWorks binder with the same name of DocuWork document.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl8
{
    class Program
    {
        static int Main(string[] args)
        {
            int api_result = 0;

            if (args.Length != 1) {
                TextWriter errorWriter = Console.Error;
                errorWriter.WriteLine("How to use : xdwsmpl8 document");
                return 0;
            }

            Xdwapi.XDW_DOCUMENT_HANDLE handle = new Xdwapi.XDW_DOCUMENT_HANDLE();
            Xdwapi.XDW_OPEN_MODE_EX mode = new Xdwapi.XDW_OPEN_MODE_EX();
            mode.Option = Xdwapi.XDW_OPEN_UPDATE;
            mode.AuthMode = Xdwapi.XDW_AUTH_NODIALOGUE;
            
            // Open the document handle of DocuWorks document.
            string in_path = Path.GetFullPath(args[0]);
            api_result = Xdwapi.XDW_OpenDocumentHandle(in_path, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Check the document type.
            api_result = check_document_type(handle);
            if (api_result < 0) {
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Set the attributes of headers, footers and page-numbers.
            api_result = set_pageform_attribute(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Update headers, footers, and page-numbers.
            api_result = Xdwapi.XDW_UpdatePageForm(handle, Xdwapi.XDW_PAGEFORM_STAY);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Reflect the changes to the document.
            api_result = Xdwapi.XDW_SaveDocument(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            // Create DocuWorks binder with the same name of DocuWork document.
            string name = Path.GetFileNameWithoutExtension(in_path);
            name += ".xbd";
            string binder_path = Path.GetFullPath(name);
            if (File.Exists(binder_path)) {
                print_error(Xdwapi.XDW_E_FILE_EXISTS);
                return 0;
            }
            api_result = Xdwapi.XDW_CreateBinder(binder_path, null);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Open the document handle of the binder.
            api_result = Xdwapi.XDW_OpenDocumentHandle(binder_path, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Insert the document to the binder.
            api_result = Xdwapi.XDW_InsertDocumentToBinder(handle, 1, in_path);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Set the attributes of headers, footers and page-numbers.
            api_result = set_pageform_attribute(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Update headers, footers, and page-numbers.
            // If the second argument is set XDW_PAGEFORM_REMOVE,
            // headers, footers and page-numbers of the internal document are removed.
            api_result = Xdwapi.XDW_UpdatePageForm(handle, Xdwapi.XDW_PAGEFORM_STAY);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Reflect the changes to the binder.
            api_result = Xdwapi.XDW_SaveDocument(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            if (api_result < 0)
                return 0;

            return 1;
        }

        static int check_document_type(Xdwapi.XDW_DOCUMENT_HANDLE handle)
        {
            Xdwapi.XDW_DOCUMENT_INFO info = new Xdwapi.XDW_DOCUMENT_INFO();
            int api_result = Xdwapi.XDW_GetDocumentInformation(handle, ref info);
            if (api_result < 0)
                return api_result;

            if (info.DocType != Xdwapi.XDW_DT_DOCUMENT)
            {
                TextWriter errorWriter = Console.Error;
                errorWriter.WriteLine("Set DocuWorks document to the file.");
                return Xdwapi.XDW_E_INVALIDARG;
            }

            return api_result;
        }

        static int set_pageform_attribute(Xdwapi.XDW_DOCUMENT_HANDLE handle)
        {
            Xdwapi.XDW_DOCUMENT_INFO info = new Xdwapi.XDW_DOCUMENT_INFO();
            int api_result = Xdwapi.XDW_GetDocumentInformation(handle, ref info);
            if (api_result < 0)
                return api_result;

            string out_string = null;
            int alignment = 0;
            if (info.DocType == Xdwapi.XDW_DT_DOCUMENT) {
                out_string = "page number # in DocuWorks document";
                // In case of DocuWorks document, set page-number at the lower left of the page.
                alignment = Xdwapi.XDW_ALIGN_LEFT;
            } else {
                out_string = "page number # in DocuWorks binder";
                // In case of DocuWorks document, set page-number at the lower right of the page.
                alignment = Xdwapi.XDW_ALIGN_RIGHT;
            }

            //Set the content of the page-number.
            api_result = Xdwapi.XDW_SetPageFormAttribute(handle, Xdwapi.XDW_PAGEFORM_PAGENUMBER, Xdwapi.XDW_ATN_Text, Xdwapi.XDW_ATYPE_STRING, out_string);
            if (api_result < 0)
                return api_result;
            
            //Set the alignment of the page-number.
            api_result = Xdwapi.XDW_SetPageFormAttribute(handle, Xdwapi.XDW_PAGEFORM_PAGENUMBER, Xdwapi.XDW_ATN_Alignment, Xdwapi.XDW_ATYPE_INT, alignment);

            return api_result;
        }

        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code)
            {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                case Xdwapi.XDW_E_FILE_EXISTS:
                    errorWriter.WriteLine("File exists.");
                    break;
                default:
                    errorWriter.WriteLine("Error.");
                    break;
            }
        }

    }
}
